// MandView.cpp : implementation of the CMandelView class
//

#include "stdafx.h"
#include "Mandel.h"

#include "MandDoc.h"
#include "MandView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// define Mandelbrot set constants:
#define CIMAX 1.2
#define CIMIN -1.2
#define CRMAX 1.0
#define CRMIN -2.0
#define NMAX 128

// colors used to create Mandelbrot pattern:
DWORD ColorTable [6] =
   {0x0000ff,  // red
    0x00ff00,  // green
    0xff0000,  // blue
    0x00ffff,  // yellow
    0xffff00,  // cyan
    0xff00ff}; // magenta

/////////////////////////////////////////////////////////////////////////////
// CMandelView

IMPLEMENT_DYNCREATE(CMandelView, CView)

BEGIN_MESSAGE_MAP(CMandelView, CView)
   //{{AFX_MSG_MAP(CMandelView)
   ON_WM_SIZE()
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMandelView construction/destruction

CMandelView::CMandelView()
{
   // TODO: add construction code here
   m_Col = 0;
}

CMandelView::~CMandelView()
{
}

BOOL CMandelView::PreCreateWindow(CREATESTRUCT& cs)
{
   // TODO: Modify the Window class or styles here by modifying
   //  the CREATESTRUCT cs

   return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMandelView drawing

void CMandelView::OnDraw(CDC* pDC)
{
   CMandelDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);

   // TODO: add draw code for native data here

   m_Col = 0;
   m_CR = (float)CRMIN;
}

/////////////////////////////////////////////////////////////////////////////
// CMandelView diagnostics

#ifdef _DEBUG
void CMandelView::AssertValid() const
{
   CView::AssertValid();
}

void CMandelView::Dump(CDumpContext& dc) const
{
   CView::Dump(dc);
}

CMandelDoc* CMandelView::GetDocument() // non-debug version is inline
{
   ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMandelDoc)));
   return (CMandelDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMandelView message handlers

void CMandelView::OnSize(UINT nType, int cx, int cy) 
{
   CView::OnSize(nType, cx, cy);
   
   // TODO: Add your message handler code here

   if (cx == 0 || cy == 0)
      return;
   
   m_ColMax = cx;
   m_RowMax = cy;

   m_DCR = (float)((CRMAX - CRMIN) / (m_ColMax-1));
   m_DCI = (float)((CIMAX - CIMIN) / (m_RowMax-1));
}

void CMandelView::DrawCol ()
{
   CClientDC ClientDC (this);
   float CI;
   int ColorVal;
   float I;
   float ISqr;
   float R;
   int Row;
   float RSqr;

   if (m_Col >= m_ColMax || GetParentFrame ()->IsIconic ())
      return;

   CI = (float)CIMAX;
   for (Row = 0; Row < m_RowMax; ++Row)
      {
      R = (float)0.0;
      I = (float)0.0;
      RSqr = (float)0.0;
      ISqr = (float)0.0;
      ColorVal = 0;
      while (ColorVal < NMAX && RSqr + ISqr < 4)
         {
         ++ColorVal;
         RSqr = R * R;
         ISqr = I * I;
         I *= R;
         I += I + CI;
         R = RSqr - ISqr + m_CR;
         }
      ClientDC.SetPixelV (m_Col, Row, ColorTable [ColorVal % 6]);
      CI -= m_DCI;
      }
   m_Col++;
   m_CR += m_DCR;
}
